@@ -20,6 +20,7 @@
 #include "src/util.h"
 #include "src/priv.h"
 #include "classes/http.h"
+#include "zend_exceptions.h"
 
 #if 0
 ZEND_DECLARE_MODULE_GLOBALS(event)
@@ -47,6 +48,9 @@ zend_class_entry *php_event_http_ce;
 zend_class_entry *php_event_http_req_ce;
 #endif
 
+static zend_class_entry *spl_ce_RuntimeException;
+zend_class_entry *php_event_exception_ce;
+
 static HashTable classes;
 
 static HashTable event_properties;
@@ -102,6 +106,35 @@ zend_module_entry event_module_entry = {
 ZEND_GET_MODULE(event)
 #endif
 
+zend_class_entry *php_event_get_exception(void)/*{{{*/
+{
+	return php_event_exception_ce;
+}/*}}}*/
+
+
+zend_class_entry *php_event_get_exception_base(int root TSRMLS_DC)/*{{{*/
+{
+#if can_handle_soft_dependency_on_SPL && defined(HAVE_SPL) && ((PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1))
+	if (!root) {
+		if (!spl_ce_RuntimeException) {
+			zend_class_entry **pce;
+
+			if (zend_hash_find(CG(class_table), "runtimeexception", sizeof("RuntimeException"), (void **) &pce) == SUCCESS) {
+				spl_ce_RuntimeException = *pce;
+				return *pce;
+			}
+		} else {
+			return spl_ce_RuntimeException;
+		}
+	}
+#endif
+#if (PHP_MAJOR_VERSION == 5) && (PHP_MINOR_VERSION < 2)
+	return zend_exception_get_default();
+#else
+	return zend_exception_get_default(TSRMLS_C);
+#endif
+}/*}}}*/
+
 
 /* {{{ Private functions */
 
@@ -601,6 +634,7 @@ static zend_object_value event_buffer_object_create(zend_class_entry *ce TSRMLS_
  * EventUtil object ctor */
 static zend_object_value event_util_object_create(zend_class_entry *ce TSRMLS_DC)
 {
+#if 0
 	php_event_abstract_object_t *obj;
 
 	/* EventUtil is a singleton. This function must never be called */
@@ -610,6 +644,17 @@ static zend_object_value event_util_object_create(zend_class_entry *ce TSRMLS_DC
 
 	return register_object(ce, (void *) obj, (zend_objects_store_dtor_t) zend_objects_destroy_object,
 			event_generic_object_free_storage TSRMLS_CC);
+#endif
+
+	zend_object *object;
+	zend_object_value value;
+
+	value = zend_objects_new(&object, ce TSRMLS_CC);
+	value.handlers = zend_get_std_object_handlers();
+
+	object_properties_init(object, ce);
+
+	return value;
 }
 /* }}} */
 
@@ -1058,6 +1103,7 @@ static HashTable *get_gc(zval *object, zval ***table, int *n TSRMLS_DC)
 static zend_always_inline void register_classes(TSRMLS_D)
 {
 	zend_class_entry *ce;
+	zend_class_entry ce_exception;
 
 	PHP_EVENT_REGISTER_CLASS("Event", event_object_create, php_event_ce, php_event_ce_functions);
 	ce = php_event_ce;
@@ -1150,6 +1196,13 @@ static zend_always_inline void register_classes(TSRMLS_D)
 			sizeof(event_ssl_context_properties), NULL);
 #endif /* HAVE_EVENT_OPENSSL_LIB */
 
+#ifdef PHP_EVENT_NS
+	INIT_NS_CLASS_ENTRY(ce_exception, PHP_EVENT_NS, "EventException", NULL);
+#else
+	INIT_CLASS_ENTRY(ce_exception, "EventException", NULL);
+#endif
+	php_event_exception_ce = zend_register_internal_class_ex(&ce_exception, php_event_get_exception_base(0 TSRMLS_CC), NULL TSRMLS_CC);
+	zend_declare_property_null(php_event_exception_ce, "errorInfo", sizeof("errorInfo") - 1, ZEND_ACC_PUBLIC TSRMLS_CC);
 }
 /* }}} */
 
@@ -1183,6 +1236,14 @@ PHP_MINIT_FUNCTION(event)
 #endif
 	object_handlers.get_gc               = get_gc;
 
+	spl_ce_RuntimeException = NULL;
+
+#ifdef PHP_EVENT_NS
+	REGISTER_STRING_CONSTANT("EVENT_NS", PHP_EVENT_NS, CONST_CS | CONST_PERSISTENT);
+#else
+	REGISTER_STRING_CONSTANT("EVENT_NS", "", CONST_CS | CONST_PERSISTENT);
+#endif
+
 	zend_hash_init(&classes, 8, NULL, NULL, 1);
 	register_classes(TSRMLS_C);
 
